@@ -5,17 +5,38 @@
 GET_PIP_CACHE_LOCATION=/opt/stack/cache/files/get-pip.py
 
 function octavia_install {
-
     setup_develop $OCTAVIA_DIR
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
         if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-            install_package qemu kpartx
-            git_clone $DISKIMAGE_BUILDER_REPO $DISKIMAGE_BUILDER_DIR $DISKIMAGE_BUILDER_BRANCH
-            sudo -H -E pip install -r $DEST/diskimage-builder/requirements.txt
+            if [[ ${DISTRO} =~ "rhel7" ]]; then
+                # Installing qemu would bring in the default OS qemu package,
+                # which is too old for Pike and later releases
+                # See https://review.openstack.org/#/c/438325 for details
+                install_package qemu-kvm
+            else
+                install_package qemu
+            fi
         fi
     fi
 }
 
+function octaviaclient_install {
+    if use_library_from_git "python-octaviaclient"; then
+        git_clone_by_name "python-octaviaclient"
+        setup_dev_lib "python-octaviaclient"
+    fi
+}
+
+function install_diskimage_builder {
+    if use_library_from_git "diskimage-builder"; then
+        GITREPO["diskimage-builder"]=$DISKIMAGE_BUILDER_REPO_URL
+        GITDIR["diskimage-builder"]=$DISKIMAGE_BUILDER_DIR
+        GITBRANCH["diskimage-builder"]=$DISKIMAGE_BUILDER_REPO_REF
+        git_clone_by_name "diskimage-builder"
+        setup_dev_lib "diskimage-builder"
+    fi
+}
+
 function set_octavia_worker_image_owner_id {
     image_id=$(openstack image list --property name=${OCTAVIA_AMP_IMAGE_NAME} -f value -c ID)
     owner_id=$(openstack image show ${image_id} -c owner -f value)
@@ -23,6 +44,8 @@ function set_octavia_worker_image_owner_id {
 }
 
 function build_octavia_worker_image {
+    # set up diskimage-builder if we need to
+    install_diskimage_builder
 
     # pull the agent code from the current code zuul has a reference to
     if [ -n "$DIB_REPOLOCATION_pip_and_virtualenv" ]; then
@@ -39,29 +62,125 @@ function build_octavia_worker_image {
     if [ "$OCTAVIA_DIB_TRACING" != "0" ]; then
         octavia_dib_tracing_arg="-x"
     fi
+    if [[ ${OCTAVIA_AMP_BASE_OS:+1} ]] ; then
+    export PARAM_OCTAVIA_AMP_BASE_OS='-i '$OCTAVIA_AMP_BASE_OS
+    fi
+    if [[ ${OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:+1} ]] ; then
+    export PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID='-d '$OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID
+    fi
+    if [[ ${OCTAVIA_AMP_IMAGE_SIZE:+1} ]] ; then
+    export PARAM_OCTAVIA_AMP_IMAGE_SIZE='-s '$OCTAVIA_AMP_IMAGE_SIZE
+    fi
     if ! [ -f $OCTAVIA_AMP_IMAGE_FILE ]; then
-        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh $octavia_dib_tracing_arg -s 2 -o $OCTAVIA_AMP_IMAGE_FILE
+        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-''} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-''} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-''}
     fi
     upload_image file://${OCTAVIA_AMP_IMAGE_FILE} $TOKEN
 
 }
 
+function _configure_octavia_apache_wsgi {
+
+    # Make sure mod_wsgi is enabled in apache
+    # This is important for multinode where other services have not yet
+    # enabled it.
+    install_apache_wsgi
+
+    local octavia_apache_conf
+    octavia_apache_conf=$(apache_site_config_for octavia)
+
+    # Use the alternate port if we are running multinode behind haproxy
+    if [ $OCTAVIA_NODE != 'standalone' ] && [ $OCTAVIA_NODE != 'api' ]; then
+        local octavia_api_port=$OCTAVIA_HA_PORT
+    else
+        local octavia_api_port=$OCTAVIA_PORT
+    fi
+    local octavia_ssl=""
+    local octavia_certfile=""
+    local octavia_keyfile=""
+    local venv_path=""
+
+    if is_ssl_enabled_service octavia; then
+        octavia_ssl="SSLEngine On"
+        octavia_certfile="SSLCertificateFile $OCTAVIA_SSL_CERT"
+        octavia_keyfile="SSLCertificateKeyFile $OCTAVIA_SSL_KEY"
+    fi
+
+    if [[ ${USE_VENV} = True ]]; then
+        venv_path="python-path=${PROJECT_VENV["octavia"]}/lib/$(python_version)/site-packages"
+    fi
+
+    sudo cp ${OCTAVIA_DIR}/devstack/files/wsgi/octavia-api.template $octavia_apache_conf
+    sudo sed -e "
+        s|%OCTAVIA_SERVICE_PORT%|$octavia_api_port|g;
+        s|%USER%|$APACHE_USER|g;
+        s|%APACHE_NAME%|$APACHE_NAME|g;
+        s|%SSLENGINE%|$octavia_ssl|g;
+        s|%SSLCERTFILE%|$octavia_certfile|g;
+        s|%SSLKEYFILE%|$octavia_keyfile|g;
+        s|%VIRTUALENV%|$venv_path|g
+        s|%APIWORKERS%|$API_WORKERS|g;
+    " -i $octavia_apache_conf
+
+}
+
+function _configure_octavia_apache_uwsgi {
+    write_uwsgi_config "$OCTAVIA_UWSGI_CONF" "$OCTAVIA_UWSGI_APP" "/$OCTAVIA_SERVICE_TYPE"
+}
+
+
+function _cleanup_octavia_apache_wsgi {
+    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+        remove_uwsgi_config "$OCTAVIA_UWSGI_CONF" "$OCTAVIA_UWSGI_APP"
+        restart_apache_server
+    else
+        sudo rm -f $(apache_site_config_for octavia)
+        restart_apache_server
+    fi
+}
+
+function _start_octavia_apache_wsgi {
+    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+        run_process o-api "$OCTAVIA_BIN_DIR/uwsgi --ini $OCTAVIA_UWSGI_CONF"
+        enable_apache_site octavia-wsgi
+    else
+        enable_apache_site octavia
+        restart_apache_server
+    fi
+}
+
+function _stop_octavia_apache_wsgi {
+    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+        disable_apache_site octavia-wsgi
+        stop_process o-api
+    else
+        disable_apache_site octavia
+        restart_apache_server
+    fi
+}
+
 function create_octavia_accounts {
-    create_service_user "octavia"
+    create_service_user $OCTAVIA
 
     local octavia_service=$(get_or_create_service "octavia" \
-        "octavia" "Octavia Service")
-    get_or_create_endpoint $octavia_service \
-        "$REGION_NAME" \
-        "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
-        "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
-        "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
+        $OCTAVIA_SERVICE_TYPE "Octavia Load Balancing Service")
+
+    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+        get_or_create_endpoint $octavia_service \
+            "$REGION_NAME" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE"
+    else
+        get_or_create_endpoint $octavia_service \
+            "$REGION_NAME" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
+    fi
 }
 
 function octavia_configure {
 
-    create_octavia_cache_dir
-
     sudo mkdir -m 755 -p $OCTAVIA_CONF_DIR
     safe_chown $STACK_USER $OCTAVIA_CONF_DIR
 
@@ -75,10 +194,10 @@ function octavia_configure {
     iniset $OCTAVIA_CONF database connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia"
 
     # Configure keystone auth_token for all users
-    configure_auth_token_middleware $OCTAVIA_CONF octavia $OCTAVIA_AUTH_CACHE_DIR
+    configure_auth_token_middleware $OCTAVIA_CONF octavia
 
     # Ensure config is set up properly for authentication as admin
-    iniset $OCTAVIA_CONF service_auth auth_url $KEYSTONE_AUTH_URI
+    iniset $OCTAVIA_CONF service_auth auth_url $OS_AUTH_URL
     iniset $OCTAVIA_CONF service_auth auth_type password
     iniset $OCTAVIA_CONF service_auth username $OCTAVIA_USERNAME
     iniset $OCTAVIA_CONF service_auth password $OCTAVIA_PASSWORD
@@ -86,7 +205,6 @@ function octavia_configure {
     iniset $OCTAVIA_CONF service_auth project_name $OCTAVIA_PROJECT_NAME
     iniset $OCTAVIA_CONF service_auth project_domain_name $OCTAVIA_PROJECT_DOMAIN_NAME
     iniset $OCTAVIA_CONF service_auth cafile $SSL_BUNDLE_FILE
-    iniset $OCTAVIA_CONF service_auth signing_dir $signing_dir
     iniset $OCTAVIA_CONF service_auth memcached_servers $SERVICE_HOST:11211
 
     # Setting other required default options
@@ -109,7 +227,11 @@ function octavia_configure {
 
     # Setting neutron request_poll_timeout
     iniset $NEUTRON_CONF octavia request_poll_timeout 3000
-    iniset $NEUTRON_CONF octavia base_url http://$SERVICE_HOST:9876
+    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+        iniadd $NEUTRON_CONF octavia base_url "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE"
+    else
+        iniadd $NEUTRON_CONF octavia base_url "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
+    fi
 
     # Uncomment other default options
     iniuncomment $OCTAVIA_CONF haproxy_amphora base_path
@@ -120,6 +242,7 @@ function octavia_configure {
     iniuncomment $OCTAVIA_CONF haproxy_amphora rest_request_read_timeout
     iniuncomment $OCTAVIA_CONF controller_worker amp_active_retries
     iniuncomment $OCTAVIA_CONF controller_worker amp_active_wait_sec
+    iniuncomment $OCTAVIA_CONF controller_worker workers
 
     # devstack optimizations for tempest runs
     iniset $OCTAVIA_CONF haproxy_amphora connection_max_retries 1500
@@ -128,6 +251,7 @@ function octavia_configure {
     iniset $OCTAVIA_CONF haproxy_amphora rest_request_read_timeout ${OCTAVIA_AMP_READ_TIMEOUT}
     iniset $OCTAVIA_CONF controller_worker amp_active_retries 100
     iniset $OCTAVIA_CONF controller_worker amp_active_wait_sec 2
+    iniset $OCTAVIA_CONF controller_worker workers 2
 
     if [[ -a $OCTAVIA_SSH_DIR ]] ; then
         rm -rf $OCTAVIA_SSH_DIR
@@ -144,10 +268,7 @@ function octavia_configure {
     fi
     iniset $OCTAVIA_CONF controller_worker amp_ssh_key_name ${OCTAVIA_AMP_SSH_KEY_NAME}
 
-    # Used to communicate with the amphora over the mgmt network, may differ from amp_ssh_key in a real deployment.
-    iniset $OCTAVIA_CONF haproxy_amphora key_path ${OCTAVIA_AMP_SSH_KEY_PATH}
-
-    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
+    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] || [ $OCTAVIA_NODE == 'api' ]; then
         recreate_database_mysql octavia
         octavia-db-manage upgrade head
     fi
@@ -172,6 +293,14 @@ function octavia_configure {
     mkdir -m755 -p $OCTAVIA_DHCLIENT_DIR
     cp $OCTAVIA_DIR/etc/dhcp/dhclient.conf $OCTAVIA_DHCLIENT_CONF
 
+    if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
+        if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+            _configure_octavia_apache_uwsgi
+        else
+            _configure_octavia_apache_wsgi
+        fi
+    fi
+
 }
 
 function create_mgmt_network_interface {
@@ -189,7 +318,7 @@ function create_mgmt_network_interface {
     # TODO(johnsom) This gets the IPv4 address, should be updated for IPv6
     MGMT_PORT_IP=$(openstack port show -f value -c fixed_ips $MGMT_PORT_ID | awk '{FS=",| "; gsub(",",""); gsub("'\''",""); for(i = 1; i <= NF; ++i) {if ($i ~ /^ip_address/) {n=index($i, "="); if (substr($i, n+1) ~ "\\.") print substr($i, n+1)}}}')
     if [[ $Q_AGENT == "openvswitch" ]]; then
-        sudo ovs-vsctl -- --may-exist add-port ${OVS_BRIDGE:-br-int} o-hm0 -- set Interface o-hm0 type=internal -- set Interface o-hm0 external-ids:iface-status=active -- set Interface o-hm0 external-ids:attached-mac=$MGMT_PORT_MAC -- set Interface o-hm0 external-ids:iface-id=$MGMT_PORT_ID
+        sudo ovs-vsctl -- --may-exist add-port ${OVS_BRIDGE:-br-int} o-hm0 -- set Interface o-hm0 type=internal -- set Interface o-hm0 external-ids:iface-status=active -- set Interface o-hm0 external-ids:attached-mac=$MGMT_PORT_MAC -- set Interface o-hm0 external-ids:iface-id=$MGMT_PORT_ID -- set Interface o-hm0 external-ids:skip_cleanup=true
     elif [[ $Q_AGENT == "linuxbridge" ]]; then
         if ! ip link show o-hm0 ; then
             sudo ip link add o-hm0 type veth peer name o-bhm0
@@ -280,7 +409,7 @@ function octavia_start {
     # Several steps in this function would more logically be in the configure function, but
     # we need nova, glance, and neutron to be running.
 
-    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ] ; then
+    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ]  && [ $OCTAVIA_NODE != 'api' ]; then
         # without the other services enabled apparently we don't have
         # credentials at this point
         TOP_DIR=$(cd $(dirname "$0") && pwd)
@@ -325,10 +454,13 @@ function octavia_start {
     if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
         set_octavia_worker_image_owner_id
     fi
-    create_amphora_flavor
 
-    create_mgmt_network_interface
-    configure_lb_mgmt_sec_grp
+    if [ $OCTAVIA_NODE != 'api' ] ; then
+        create_mgmt_network_interface
+
+        create_amphora_flavor
+        configure_lb_mgmt_sec_grp
+    fi
 
     iniset $OCTAVIA_CONF controller_worker amp_image_tag ${OCTAVIA_AMP_IMAGE_TAG}
 
@@ -349,15 +481,32 @@ function octavia_start {
         iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
     fi
 
-    run_process $OCTAVIA_API  "$OCTAVIA_API_BINARY $OCTAVIA_API_ARGS"
+    if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
+        _start_octavia_apache_wsgi
+    else
+        run_process $OCTAVIA_API  "$OCTAVIA_API_BINARY $OCTAVIA_API_ARGS"
+    fi
+
     run_process $OCTAVIA_CONSUMER  "$OCTAVIA_CONSUMER_BINARY $OCTAVIA_CONSUMER_ARGS"
     run_process $OCTAVIA_HOUSEKEEPER  "$OCTAVIA_HOUSEKEEPER_BINARY $OCTAVIA_HOUSEKEEPER_ARGS"
     run_process $OCTAVIA_HEALTHMANAGER  "$OCTAVIA_HEALTHMANAGER_BINARY $OCTAVIA_HEALTHMANAGER_ARGS"
 
+    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
+        add_load-balancer_roles
+    fi
+
 }
 
 function octavia_stop {
     # octavia-specific stop actions
+    if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
+        _stop_octavia_apache_wsgi
+    else
+        stop_process $OCTAVIA_API
+    fi
+    stop_process $OCTAVIA_CONSUMER
+    stop_process $OCTAVIA_HOUSEKEEPER
+    stop_process $OCTAVIA_HEALTHMANAGER
 
     # Kill dhclient process started for o-hm0 interface
     pids=$(ps aux | awk '/o-hm0/ { print $2 }')
@@ -369,13 +518,6 @@ function octavia_stop {
     fi
 }
 
-function octavia_configure_common {
-    if is_service_enabled $OCTAVIA_SERVICE && [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE = 'standalone' ] ; then
-        inicomment $NEUTRON_LBAAS_CONF service_providers service_provider
-        iniadd $NEUTRON_LBAAS_CONF service_providers service_provider $OCTAVIA_SERVICE_PROVIDER
-    fi
-}
-
 function octavia_cleanup {
 
     if [ ${OCTAVIA_AMP_IMAGE_NAME}x != x ] ; then
@@ -398,15 +540,20 @@ function octavia_cleanup {
             openstack keypair delete ${OCTAVIA_AMP_SSH_KEY_NAME}
         fi
     fi
+    if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
+        _cleanup_octavia_apache_wsgi
+    fi
 
     sudo rm -rf $NOVA_STATE_PATH $NOVA_AUTH_CACHE_DIR
 }
 
-# create_octavia_cache_dir() - Part of the configure_octavia() process
-function create_octavia_cache_dir {
-    # Create cache dir
-    sudo install -d -o $STACK_USER $OCTAVIA_AUTH_CACHE_DIR
-    rm -f $OCTAVIA_AUTH_CACHE_DIR/*
+function add_load-balancer_roles {
+    openstack role create load-balancer_observer
+    openstack role create load-balancer_global_observer
+    openstack role create load-balancer_member
+    openstack role create load-balancer_admin
+    openstack role create load-balancer_quota_admin
+    openstack role add --user demo --project demo load-balancer_member
 }
 
 # check for service enabled
@@ -427,12 +574,12 @@ if is_service_enabled $OCTAVIA; then
         # Perform installation of service source
         echo_summary "Installing octavia"
         octavia_install
+        octaviaclient_install
 
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         # Configure after the other layer 1 and 2 services have been configured
         # TODO: need to make sure this runs after LBaaS V2 configuration
         echo_summary "Configuring octavia"
-        # octavia_configure_common
         octavia_configure
 
     elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
