@@ -27,7 +27,6 @@ from octavia.controller.worker.flows import member_flows
 from octavia.controller.worker.flows import pool_flows
 from octavia.db import api as db_apis
 from octavia.db import repositories as repo
-from octavia.i18n import _LE, _LI
 
 from oslo_config import cfg
 from oslo_utils import excutils
@@ -80,8 +79,11 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         :returns: amphora_id
         """
-        create_amp_tf = self._taskflow_load(self._amphora_flows.
-                                            get_create_amphora_flow())
+        create_amp_tf = self._taskflow_load(
+            self._amphora_flows.get_create_amphora_flow(),
+            store={constants.BUILD_TYPE_PRIORITY:
+                   constants.LB_CREATE_SPARES_POOL_PRIORITY}
+        )
         with tf_logging.DynamicLoggingListener(
                 create_amp_tf, log=LOG,
                 hide_inputs_outputs_of=self._exclude_result_logging_tasks):
@@ -106,7 +108,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             delete_amp_tf.run()
 
-    def create_health_monitor(self, pool_id):
+    def create_health_monitor(self, health_monitor_id):
         """Creates a health monitor.
 
         :param pool_id: ID of the pool to create a health monitor on
@@ -114,25 +116,24 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :raises NoSuitablePool: Unable to find the node pool
         """
         health_mon = self._health_mon_repo.get(db_apis.get_session(),
-                                               pool_id=pool_id)
+                                               id=health_monitor_id)
 
-        listeners = health_mon.pool.listeners
-        health_mon.pool.health_monitor = health_mon
-        load_balancer = health_mon.pool.load_balancer
+        pool = health_mon.pool
+        listeners = pool.listeners
+        pool.health_monitor = health_mon
+        load_balancer = pool.load_balancer
 
-        create_hm_tf = self._taskflow_load(self._health_monitor_flows.
-                                           get_create_health_monitor_flow(),
-                                           store={constants.HEALTH_MON:
-                                                  health_mon,
-                                                  constants.LISTENERS:
-                                                      listeners,
-                                                  constants.LOADBALANCER:
-                                                      load_balancer})
+        create_hm_tf = self._taskflow_load(
+            self._health_monitor_flows.get_create_health_monitor_flow(),
+            store={constants.HEALTH_MON: health_mon,
+                   constants.POOL: pool,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
         with tf_logging.DynamicLoggingListener(create_hm_tf,
                                                log=LOG):
             create_hm_tf.run()
 
-    def delete_health_monitor(self, pool_id):
+    def delete_health_monitor(self, health_monitor_id):
         """Deletes a health monitor.
 
         :param pool_id: ID of the pool to delete its health monitor
@@ -140,21 +141,23 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :raises HMNotFound: The referenced health monitor was not found
         """
         health_mon = self._health_mon_repo.get(db_apis.get_session(),
-                                               pool_id=pool_id)
+                                               id=health_monitor_id)
 
-        listeners = health_mon.pool.listeners
-        load_balancer = health_mon.pool.load_balancer
+        pool = health_mon.pool
+        listeners = pool.listeners
+        load_balancer = pool.load_balancer
 
         delete_hm_tf = self._taskflow_load(
             self._health_monitor_flows.get_delete_health_monitor_flow(),
-            store={constants.HEALTH_MON: health_mon, constants.POOL_ID:
-                   pool_id, constants.LISTENERS: listeners,
+            store={constants.HEALTH_MON: health_mon,
+                   constants.POOL: pool,
+                   constants.LISTENERS: listeners,
                    constants.LOADBALANCER: load_balancer})
         with tf_logging.DynamicLoggingListener(delete_hm_tf,
                                                log=LOG):
             delete_hm_tf.run()
 
-    def update_health_monitor(self, pool_id, health_monitor_updates):
+    def update_health_monitor(self, health_monitor_id, health_monitor_updates):
         """Updates a health monitor.
 
         :param pool_id: ID of the pool to have it's health monitor updated
@@ -163,22 +166,20 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :raises HMNotFound: The referenced health monitor was not found
         """
         health_mon = self._health_mon_repo.get(db_apis.get_session(),
-                                               pool_id=pool_id)
+                                               id=health_monitor_id)
 
-        listeners = health_mon.pool.listeners
-        health_mon.pool.health_monitor = health_mon
-        load_balancer = health_mon.pool.load_balancer
+        pool = health_mon.pool
+        listeners = pool.listeners
+        pool.health_monitor = health_mon
+        load_balancer = pool.load_balancer
 
-        update_hm_tf = self._taskflow_load(self._health_monitor_flows.
-                                           get_update_health_monitor_flow(),
-                                           store={constants.HEALTH_MON:
-                                                  health_mon,
-                                                  constants.LISTENERS:
-                                                      listeners,
-                                                  constants.LOADBALANCER:
-                                                      load_balancer,
-                                                  constants.UPDATE_DICT:
-                                                      health_monitor_updates})
+        update_hm_tf = self._taskflow_load(
+            self._health_monitor_flows.get_update_health_monitor_flow(),
+            store={constants.HEALTH_MON: health_mon,
+                   constants.POOL: pool,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer,
+                   constants.UPDATE_DICT: health_monitor_updates})
         with tf_logging.DynamicLoggingListener(update_hm_tf,
                                                log=LOG):
             update_hm_tf.run()
@@ -261,7 +262,9 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :raises NoSuitableAmphoraException: Unable to allocate an Amphora.
         """
 
-        store = {constants.LOADBALANCER_ID: load_balancer_id}
+        store = {constants.LOADBALANCER_ID: load_balancer_id,
+                 constants.BUILD_TYPE_PRIORITY:
+                 constants.LB_CREATE_NORMAL_PRIORITY}
 
         topology = CONF.controller_worker.loadbalancer_topology
 
@@ -312,7 +315,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         lb = self._lb_repo.get(db_apis.get_session(),
                                id=load_balancer_id)
-        listeners = self._listener_repo.get_all(
+        listeners, _ = self._listener_repo.get_all(
             db_apis.get_session(),
             load_balancer_id=load_balancer_id)
 
@@ -335,9 +338,9 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         member = self._member_repo.get(db_apis.get_session(),
                                        id=member_id)
-
-        listeners = member.pool.listeners
-        load_balancer = member.pool.load_balancer
+        pool = member.pool
+        listeners = pool.listeners
+        load_balancer = pool.load_balancer
 
         create_member_tf = self._taskflow_load(self._member_flows.
                                                get_create_member_flow(),
@@ -345,7 +348,8 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                       constants.LISTENERS:
                                                           listeners,
                                                       constants.LOADBALANCER:
-                                                          load_balancer})
+                                                          load_balancer,
+                                                      constants.POOL: pool})
         with tf_logging.DynamicLoggingListener(create_member_tf,
                                                log=LOG):
             create_member_tf.run()
@@ -359,14 +363,15 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         member = self._member_repo.get(db_apis.get_session(),
                                        id=member_id)
-
-        listeners = member.pool.listeners
-        load_balancer = member.pool.load_balancer
+        pool = member.pool
+        listeners = pool.listeners
+        load_balancer = pool.load_balancer
 
         delete_member_tf = self._taskflow_load(
             self._member_flows.get_delete_member_flow(),
             store={constants.MEMBER: member, constants.LISTENERS: listeners,
-                   constants.LOADBALANCER: load_balancer})
+                   constants.LOADBALANCER: load_balancer, constants.POOL: pool}
+        )
         with tf_logging.DynamicLoggingListener(delete_member_tf,
                                                log=LOG):
             delete_member_tf.run()
@@ -381,9 +386,9 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         member = self._member_repo.get(db_apis.get_session(),
                                        id=member_id)
-
-        listeners = member.pool.listeners
-        load_balancer = member.pool.load_balancer
+        pool = member.pool
+        listeners = pool.listeners
+        load_balancer = pool.load_balancer
 
         update_member_tf = self._taskflow_load(self._member_flows.
                                                get_update_member_flow(),
@@ -392,6 +397,8 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                           listeners,
                                                       constants.LOADBALANCER:
                                                           load_balancer,
+                                                      constants.POOL:
+                                                          pool,
                                                       constants.UPDATE_DICT:
                                                           member_updates})
         with tf_logging.DynamicLoggingListener(update_member_tf,
@@ -617,12 +624,25 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         try:
             amp = self._amphora_repo.get(db_apis.get_session(),
                                          id=amphora_id)
+            stored_params = {constants.FAILED_AMPHORA: amp,
+                             constants.LOADBALANCER_ID: amp.load_balancer_id,
+                             constants.BUILD_TYPE_PRIORITY:
+                                 constants.LB_CREATE_FAILOVER_PRIORITY}
+
+            # if we run with anti-affinity we need to set the server group
+            # as well
+            if CONF.nova.enable_anti_affinity:
+                lb = self._amphora_repo.get_all_lbs_on_amphora(
+                    db_apis.get_session(), amp.id)
+                if lb:
+                    stored_params[constants.SERVER_GROUP_ID] = (
+                        lb[0].server_group_id)
 
             failover_amphora_tf = self._taskflow_load(
-                self._amphora_flows.get_failover_flow(role=amp.role,
-                                                      status=amp.status),
-                store={constants.FAILED_AMPHORA: amp,
-                       constants.LOADBALANCER_ID: amp.load_balancer_id})
+                self._amphora_flows.get_failover_flow(
+                    role=amp.role,
+                    status=amp.status),
+                store=stored_params)
             with tf_logging.DynamicLoggingListener(
                     failover_amphora_tf, log=LOG,
                     hide_inputs_outputs_of=self._exclude_result_logging_tasks):
@@ -631,7 +651,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         except Exception as e:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failover exception: %s") % e)
+                LOG.error("Failover exception: %s", e)
 
     def amphora_cert_rotation(self, amphora_id):
         """Perform cert rotation for an amphora.
@@ -643,8 +663,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         amp = self._amphora_repo.get(db_apis.get_session(),
                                      id=amphora_id)
-        LOG.info(_LI("Start amphora cert rotation, amphora's id is: %s")
-                 % amp.id)
+        LOG.info("Start amphora cert rotation, amphora's id is: %s", amp.id)
 
         certrotation_amphora_tf = self._taskflow_load(
             self._amphora_flows.cert_rotate_amphora_flow(),
