@@ -31,7 +31,7 @@ class TestHaproxyCfg(base.TestCase):
 
     def test_render_template_tls(self):
         fe = ("frontend sample_listener_id_1\n"
-              "    option tcplog\n"
+              "    option httplog\n"
               "    maxconn 98\n"
               "    redirect scheme https if !{ ssl_fc }\n"
               "    bind 10.0.0.2:443 "
@@ -44,7 +44,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    fullconn 98\n"
@@ -69,7 +69,7 @@ class TestHaproxyCfg(base.TestCase):
 
     def test_render_template_tls_no_sni(self):
         fe = ("frontend sample_listener_id_1\n"
-              "    option tcplog\n"
+              "    option httplog\n"
               "    maxconn 98\n"
               "    redirect scheme https if !{ ssl_fc }\n"
               "    bind 10.0.0.2:443 "
@@ -81,7 +81,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    fullconn 98\n"
@@ -109,7 +109,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    fullconn 98\n"
@@ -126,7 +126,31 @@ class TestHaproxyCfg(base.TestCase):
             sample_configs.sample_base_expected_config(backend=be),
             rendered_obj)
 
-    def test_render_template_https(self):
+    def test_render_template_member_monitor_addr_port(self):
+        be = ("backend sample_pool_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn 98\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "addr 192.168.1.1 port 9000 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "addr 192.168.1.1 port 9000 "
+              "cookie sample_member_id_2\n\n")
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
+            sample_configs.sample_listener_tuple(monitor_ip_port=True))
+        self.assertEqual(
+            sample_configs.sample_base_expected_config(backend=be),
+            rendered_obj)
+
+    def test_render_template_https_real_monitor(self):
         fe = ("frontend sample_listener_id_1\n"
               "    option tcplog\n"
               "    maxconn 98\n"
@@ -137,9 +161,34 @@ class TestHaproxyCfg(base.TestCase):
               "    mode tcp\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
+              "    fullconn 98\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check check-ssl verify none inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check check-ssl verify none inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2\n\n")
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
+            sample_configs.sample_listener_tuple(proto='HTTPS'))
+        self.assertEqual(sample_configs.sample_base_expected_config(
+            frontend=fe, backend=be), rendered_obj)
+
+    def test_render_template_https_hello_monitor(self):
+        fe = ("frontend sample_listener_id_1\n"
+              "    option tcplog\n"
+              "    maxconn 98\n"
+              "    bind 10.0.0.2:443\n"
+              "    mode tcp\n"
+              "    default_backend sample_pool_id_1\n\n")
+        be = ("backend sample_pool_id_1\n"
+              "    mode tcp\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
               "    option ssl-hello-chk\n"
               "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 "
@@ -150,7 +199,8 @@ class TestHaproxyCfg(base.TestCase):
               "cookie sample_member_id_2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs.sample_amphora_tuple(),
-            sample_configs.sample_listener_tuple(proto='HTTPS'))
+            sample_configs.sample_listener_tuple(
+                proto='HTTPS', monitor_proto='TLS-HELLO'))
         self.assertEqual(sample_configs.sample_base_expected_config(
             frontend=fe, backend=be), rendered_obj)
 
@@ -232,7 +282,7 @@ class TestHaproxyCfg(base.TestCase):
               "    balance roundrobin\n"
               "    stick-table type ip size 10k\n"
               "    stick on src\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    fullconn 98\n"
@@ -255,7 +305,7 @@ class TestHaproxyCfg(base.TestCase):
               "    stick-table type string len 64 size 10k\n"
               "    stick store-response res.cook(JSESSIONID)\n"
               "    stick match req.cook(JSESSIONID)\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    fullconn 98\n"
@@ -274,7 +324,7 @@ class TestHaproxyCfg(base.TestCase):
 
     def test_render_template_l7policies(self):
         fe = ("frontend sample_listener_id_1\n"
-              "    option tcplog\n"
+              "    option httplog\n"
               "    maxconn 98\n"
               "    bind 10.0.0.2:80\n"
               "    mode http\n"
@@ -296,7 +346,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    fullconn 98\n"
@@ -309,7 +359,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /healthmon.html\n"
               "    http-check expect rstatus 418\n"
               "    fullconn 98\n"
@@ -326,7 +376,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    option forwardfor\n"
@@ -350,7 +400,7 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    timeout check 31\n"
+              "    timeout check 31s\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    option forwardfor\n"
@@ -371,6 +421,27 @@ class TestHaproxyCfg(base.TestCase):
             sample_configs.sample_base_expected_config(backend=be),
             rendered_obj)
 
+    def test_render_template_pool_proxy_protocol(self):
+        be = ("backend sample_pool_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    fullconn 98\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1 send-proxy\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2 send-proxy\n\n")
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
+            sample_configs.sample_listener_tuple(
+                be_proto='PROXY'))
+        self.assertEqual(
+            sample_configs.sample_base_expected_config(backend=be),
+            rendered_obj)
+
     def test_transform_session_persistence(self):
         in_persistence = sample_configs.sample_session_persistence_tuple()
         ret = self.jinja_cfg._transform_session_persistence(in_persistence)
@@ -438,6 +509,12 @@ class TestHaproxyCfg(base.TestCase):
         ret = self.jinja_cfg._transform_l7policy(in_l7policy)
         self.assertEqual(sample_configs.RET_L7POLICY_2, ret)
 
+    def test_transform_l7policy_disabled_rule(self):
+        in_l7policy = sample_configs.sample_l7policy_tuple(
+            'sample_l7policy_id_6', sample_policy=6)
+        ret = self.jinja_cfg._transform_l7policy(in_l7policy)
+        self.assertEqual(sample_configs.RET_L7POLICY_6, ret)
+
     def test_escape_haproxy_config_string(self):
         self.assertEqual(self.jinja_cfg._escape_haproxy_config_string(
             'string_with_none'), 'string_with_none')
