@@ -35,6 +35,7 @@ class TestBaseTaskFlowEngine(base.TestCase):
         conf = oslo_fixture.Config(cfg.CONF)
         conf.config(group="task_flow", max_workers=MAX_WORKERS)
         conf.config(group="task_flow", engine='TESTENGINE')
+        conf.config(group="task_flow", disable_revert=True)
         super(TestBaseTaskFlowEngine, self).setUp()
 
     @mock.patch('concurrent.futures.ThreadPoolExecutor',
@@ -59,7 +60,8 @@ class TestBaseTaskFlowEngine(base.TestCase):
         tf_engines.load.assert_called_once_with(
             'TEST',
             engine_conf='TESTENGINE',
-            executor='TESTEXECUTOR')
+            executor='TESTEXECUTOR',
+            never_resolve=True)
 
         _engine_mock.compile.assert_called_once_with()
         _engine_mock.prepare.assert_called_once_with()
