@@ -16,6 +16,7 @@ import random
 
 import mock
 from oslo_config import cfg
+from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
 import six
 import sqlalchemy
@@ -23,18 +24,28 @@ import sqlalchemy
 from octavia.common import constants
 from octavia.common import data_models
 from octavia.controller.healthmanager import update_db
+from octavia.db import models as db_models
 from octavia.tests.unit import base
 
 
+class TestException(Exception):
+
+    def __init__(self, value):
+        self.value = value
+
+    def __str__(self):
+        return repr(self.value)
+
+
 class TestUpdateHealthDb(base.TestCase):
     FAKE_UUID_1 = uuidutils.generate_uuid()
 
     def setUp(self):
         super(TestUpdateHealthDb, self).setUp()
 
-        cfg.CONF.set_override(group='health_manager',
-                              name='event_streamer_driver',
-                              override='queue_event_streamer')
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        conf.config(group="health_manager",
+                    event_streamer_driver='queue_event_streamer')
 
         session_patch = mock.patch('octavia.db.api.get_session')
         self.addCleanup(session_patch.stop)
@@ -102,7 +113,7 @@ class TestUpdateHealthDb(base.TestCase):
         self.assertTrue(lb.operating_status.lower.called)
         self.assertTrue(self.loadbalancer_repo.update.called)
 
-    def test_update_health_Online(self):
+    def test_update_health_replace_error(self):
 
         health = {
             "id": self.FAKE_UUID_1,
@@ -116,7 +127,30 @@ class TestUpdateHealthDb(base.TestCase):
             }
         }
 
-        self.mock_session.return_value = 'blah'
+        session_mock = mock.MagicMock()
+        session_mock.commit.side_effect = TestException('boom')
+        self.mock_session.return_value = session_mock
+
+        self.assertRaises(TestException, self.hm.update_health, health)
+        self.assertTrue(self.amphora_health_repo.replace.called)
+        session_mock.rollback.assert_called_once()
+
+    def test_update_health_online(self):
+
+        health = {
+            "id": self.FAKE_UUID_1,
+            "listeners": {
+                "listener-id-1": {"status": constants.OPEN, "pools": {
+                    "pool-id-1": {"status": constants.UP,
+                                  "members": {"member-id-1": constants.UP}
+                                  }
+                }
+                }
+            }
+        }
+
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
 
         self.hm.update_health(health)
         self.assertTrue(self.amphora_health_repo.replace.called)
@@ -126,22 +160,130 @@ class TestUpdateHealthDb(base.TestCase):
                 health.get('listeners', {})):
 
             self.listener_repo.update.assert_any_call(
-                'blah', listener_id, operating_status=constants.ONLINE)
+                session_mock, listener_id, operating_status=constants.ONLINE)
 
             for pool_id, pool in six.iteritems(listener.get('pools', {})):
 
                 self.hm.pool_repo.update.assert_any_call(
-                    'blah', pool_id, operating_status=constants.ONLINE)
+                    session_mock, pool_id, operating_status=constants.ONLINE)
 
                 for member_id, member in six.iteritems(
                         pool.get('members', {})):
                     self.member_repo.update.assert_any_call(
-                        'blah', member_id,
+                        session_mock, member_id,
                         operating_status=constants.ONLINE)
 
+        # If the listener count is wrong, make sure we don't update
         self.hm.listener_repo.count.return_value = 2
+        self.amphora_health_repo.replace.reset_mock()
 
         self.hm.update_health(health)
+        self.assertTrue(not self.amphora_health_repo.replace.called)
+
+    def test_update_health_member_drain(self):
+
+        health = {
+            "id": self.FAKE_UUID_1,
+            "listeners": {
+                "listener-id-1": {
+                    "status": constants.OPEN,
+                    "pools": {
+                        "pool-id-1": {
+                            "status": constants.UP,
+                            "members": {"member-id-1": constants.DRAIN}}}}}}
+
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
+
+        self.hm.update_health(health)
+        self.assertTrue(self.amphora_health_repo.replace.called)
+
+        # test listener, member
+        for listener_id, listener in six.iteritems(
+                health.get('listeners', {})):
+
+            self.listener_repo.update.assert_any_call(
+                session_mock, listener_id, operating_status=constants.ONLINE)
+
+            for pool_id, pool in six.iteritems(listener.get('pools', {})):
+
+                self.hm.pool_repo.update.assert_any_call(
+                    session_mock, pool_id, operating_status=constants.ONLINE)
+
+                for member_id, member in six.iteritems(
+                        pool.get('members', {})):
+
+                    self.member_repo.update.assert_any_call(
+                        session_mock, member_id,
+                        operating_status=constants.DRAINING)
+
+    def test_update_health_member_maint(self):
+
+        health = {
+            "id": self.FAKE_UUID_1,
+            "listeners": {
+                "listener-id-1": {
+                    "status": constants.OPEN,
+                    "pools": {
+                        "pool-id-1": {
+                            "status": constants.UP,
+                            "members": {"member-id-1": constants.MAINT}}}}}}
+
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
+
+        self.hm.update_health(health)
+        self.assertTrue(self.amphora_health_repo.replace.called)
+
+        # test listener, member
+        for listener_id, listener in six.iteritems(
+                health.get('listeners', {})):
+
+            self.listener_repo.update.assert_any_call(
+                session_mock, listener_id, operating_status=constants.ONLINE)
+
+            for pool_id, pool in six.iteritems(listener.get('pools', {})):
+
+                self.hm.pool_repo.update.assert_any_call(
+                    session_mock, pool_id, operating_status=constants.ONLINE)
+
+                for member_id, member in six.iteritems(
+                        pool.get('members', {})):
+
+                    self.member_repo.update.assert_any_call(
+                        session_mock, member_id,
+                        operating_status=constants.OFFLINE)
+
+    def test_update_health_member_unknown(self):
+
+        health = {
+            "id": self.FAKE_UUID_1,
+            "listeners": {
+                "listener-id-1": {
+                    "status": constants.OPEN,
+                    "pools": {
+                        "pool-id-1": {
+                            "status": constants.UP,
+                            "members": {"member-id-1": "blah"}}}}}}
+
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
+
+        self.hm.update_health(health)
+        self.assertTrue(self.amphora_health_repo.replace.called)
+
+        # test listener, member
+        for listener_id, listener in six.iteritems(
+                health.get('listeners', {})):
+
+            self.listener_repo.update.assert_any_call(
+                session_mock, listener_id, operating_status=constants.ONLINE)
+
+            for pool_id, pool in six.iteritems(listener.get('pools', {})):
+
+                self.hm.pool_repo.update.assert_any_call(
+                    session_mock, pool_id, operating_status=constants.ONLINE)
+                self.assertTrue(not self.member_repo.update.called)
 
     def test_update_health_member_down(self):
 
@@ -157,7 +299,8 @@ class TestUpdateHealthDb(base.TestCase):
             }
         }
 
-        self.mock_session.return_value = 'blah'
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
 
         self.hm.update_health(health)
         self.assertTrue(self.amphora_health_repo.replace.called)
@@ -167,24 +310,20 @@ class TestUpdateHealthDb(base.TestCase):
                 health.get('listeners', {})):
 
             self.listener_repo.update.assert_any_call(
-                'blah', listener_id, operating_status=constants.ONLINE)
+                session_mock, listener_id, operating_status=constants.ONLINE)
 
             for pool_id, pool in six.iteritems(listener.get('pools', {})):
 
                 self.hm.pool_repo.update.assert_any_call(
-                    'blah', pool_id, operating_status=constants.DEGRADED)
+                    session_mock, pool_id, operating_status=constants.DEGRADED)
 
                 for member_id, member in six.iteritems(
                         pool.get('members', {})):
 
                     self.member_repo.update.assert_any_call(
-                        'blah', member_id,
+                        session_mock, member_id,
                         operating_status=constants.ERROR)
 
-        self.hm.listener_repo.count.return_value = 2
-
-        self.hm.update_health(health)
-
     def test_update_health_member_no_check(self):
 
         health = {
@@ -200,7 +339,8 @@ class TestUpdateHealthDb(base.TestCase):
             }
         }
 
-        self.mock_session.return_value = 'blah'
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
 
         self.hm.update_health(health)
         self.assertTrue(self.amphora_health_repo.replace.called)
@@ -210,23 +350,66 @@ class TestUpdateHealthDb(base.TestCase):
                 health.get('listeners', {})):
 
             self.listener_repo.update.assert_any_call(
-                'blah', listener_id, operating_status=constants.ONLINE)
+                session_mock, listener_id, operating_status=constants.ONLINE)
 
             for pool_id, pool in six.iteritems(listener.get('pools', {})):
 
                 self.hm.pool_repo.update.assert_any_call(
-                    'blah', pool_id, operating_status=constants.ONLINE)
+                    session_mock, pool_id, operating_status=constants.ONLINE)
 
                 for member_id, member in six.iteritems(
                         pool.get('members', {})):
 
                     self.member_repo.update.assert_any_call(
-                        'blah', member_id,
+                        session_mock, member_id,
                         operating_status=constants.NO_MONITOR)
 
-        self.hm.listener_repo.count.return_value = 2
+    def test_update_health_member_admin_down(self):
 
+        health = {
+            "id": self.FAKE_UUID_1,
+            "listeners": {
+                "listener-id-1": {
+                    "status": constants.OPEN,
+                    "pools": {
+                        "pool-id-1": {
+                            "status": constants.UP,
+                            "members": {
+                                "member-id-1": constants.UP}}}}}}
+
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
+
+        mock_pool = mock.Mock()
+        mock_member1 = mock.Mock()
+        mock_member1.id = 'member-id-1'
+        mock_member2 = mock.Mock()
+        mock_member2.id = 'member-id-2'
+        mock_pool.members = [mock_member1, mock_member2]
+        self.pool_repo.get.return_value = mock_pool
         self.hm.update_health(health)
+        self.assertTrue(self.amphora_health_repo.replace.called)
+
+        # test listener, member
+        for listener_id, listener in six.iteritems(
+                health.get('listeners', {})):
+
+            self.listener_repo.update.assert_any_call(
+                session_mock, listener_id, operating_status=constants.ONLINE)
+
+            for pool_id, pool in six.iteritems(listener.get('pools', {})):
+
+                self.hm.pool_repo.update.assert_any_call(
+                    session_mock, pool_id, operating_status=constants.ONLINE)
+
+                for member_id, member in six.iteritems(
+                        pool.get('members', {})):
+
+                    self.member_repo.update.assert_any_call(
+                        session_mock, member_id,
+                        operating_status=constants.ONLINE)
+        self.member_repo.update.assert_any_call(
+            session_mock, mock_member2.id, operating_status=constants.OFFLINE)
 
     def test_update_health_list_full_member_down(self):
 
@@ -242,7 +425,8 @@ class TestUpdateHealthDb(base.TestCase):
             }
         }
 
-        self.mock_session.return_value = 'blah'
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
 
         self.hm.update_health(health)
         self.assertTrue(self.amphora_health_repo.replace.called)
@@ -252,25 +436,27 @@ class TestUpdateHealthDb(base.TestCase):
                 health.get('listeners', {})):
 
             self.listener_repo.update.assert_any_call(
-                'blah', listener_id, operating_status=constants.DEGRADED)
+                session_mock, listener_id, operating_status=constants.DEGRADED)
 
             for pool_id, pool in six.iteritems(listener.get('pools', {})):
 
                 self.hm.pool_repo.update.assert_any_call(
-                    'blah', pool_id, operating_status=constants.DEGRADED)
+                    session_mock, pool_id, operating_status=constants.DEGRADED)
 
                 for member_id, member in six.iteritems(
                         pool.get('members', {})):
 
                     self.member_repo.update.assert_any_call(
-                        'blah', member_id,
+                        session_mock, member_id,
                         operating_status=constants.ERROR)
 
         self.hm.listener_repo.count.return_value = 2
+        self.amphora_health_repo.replace.reset_mock()
 
         self.hm.update_health(health)
+        self.assertTrue(not self.amphora_health_repo.replace.called)
 
-    def test_update_health_Error(self):
+    def test_update_health_error(self):
 
         health = {
             "id": self.FAKE_UUID_1,
@@ -284,7 +470,8 @@ class TestUpdateHealthDb(base.TestCase):
             }
         }
 
-        self.mock_session.return_value = 'blah'
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
 
         self.hm.update_health(health)
         self.assertTrue(self.amphora_health_repo.replace.called)
@@ -294,21 +481,22 @@ class TestUpdateHealthDb(base.TestCase):
                 health.get('listeners', {})):
 
             self.listener_repo.update.assert_any_call(
-                'blah', listener_id, operating_status=constants.ONLINE)
+                session_mock, listener_id, operating_status=constants.ONLINE)
 
             for pool_id, pool in six.iteritems(listener.get('pools', {})):
 
                 self.hm.pool_repo.update.assert_any_call(
-                    'blah', pool_id, operating_status=constants.ERROR)
+                    session_mock, pool_id, operating_status=constants.ERROR)
 
                 for member_id, member in six.iteritems(
                         pool.get('members', {})):
 
                     self.member_repo.update.assert_any_call(
-                        'blah', member_id, operating_status=constants.ERROR)
+                        session_mock, member_id,
+                        operating_status=constants.ERROR)
 
     # Test the logic code paths
-    def test_update_health_Full(self):
+    def test_update_health_full(self):
 
         health = {
             "id": self.FAKE_UUID_1,
@@ -332,11 +520,23 @@ class TestUpdateHealthDb(base.TestCase):
                                   }
                 }
                 },
-                "listener-id-4": {"status": "bogus", "pools": {
-                    "pool-id-4": {"status": "bogus",
-                                  "members": {"member-id-4": "bogus"}
-                                  }
-                }
+                "listener-id-4": {
+                    "status": constants.OPEN,
+                    "pools": {
+                        "pool-id-4": {
+                            "status": constants.UP,
+                            "members": {"member-id-4": constants.DRAINING}
+                        }
+                    }
+                },
+                "listener-id-5": {
+                    "status": "bogus",
+                    "pools": {
+                        "pool-id-5": {
+                            "status": "bogus",
+                            "members": {"member-id-5": "bogus"}
+                        }
+                    }
                 }
             }
         }
@@ -356,9 +556,11 @@ class TestUpdateHealthDb(base.TestCase):
             'blah', "pool-id-2", operating_status=constants.ONLINE)
         self.pool_repo.update.assert_any_call(
             'blah', "pool-id-3", operating_status=constants.DEGRADED)
+        self.pool_repo.update.assert_any_call(
+            'blah', "pool-id-4", operating_status=constants.ONLINE)
 
     # Test code paths where objects are not found in the database
-    def test_update_health_Not_Found(self):
+    def test_update_health_not_found(self):
 
         health = {
             "id": self.FAKE_UUID_1,
@@ -381,7 +583,8 @@ class TestUpdateHealthDb(base.TestCase):
         self.hm.loadbalancer_repo.update.side_effect = (
             [sqlalchemy.orm.exc.NoResultFound])
 
-        self.mock_session.return_value = 'blah'
+        session_mock = mock.MagicMock()
+        self.mock_session.return_value = session_mock
 
         self.hm.update_health(health)
         self.assertTrue(self.amphora_health_repo.replace.called)
@@ -391,18 +594,18 @@ class TestUpdateHealthDb(base.TestCase):
                 health.get('listeners', {})):
 
             self.listener_repo.update.assert_any_call(
-                'blah', listener_id, operating_status=constants.ONLINE)
+                session_mock, listener_id, operating_status=constants.ONLINE)
 
             for pool_id, pool in six.iteritems(listener.get('pools', {})):
 
                 self.hm.pool_repo.update.assert_any_call(
-                    'blah', pool_id, operating_status=constants.ONLINE)
+                    session_mock, pool_id, operating_status=constants.ONLINE)
 
                 for member_id, member in six.iteritems(
                         pool.get('members', {})):
 
                     self.member_repo.update.assert_any_call(
-                        'blah', member_id,
+                        session_mock, member_id,
                         operating_status=constants.ONLINE)
 
     def test_update_health_no_status_change(self):
@@ -444,11 +647,64 @@ class TestUpdateHealthDb(base.TestCase):
         self.pool_repo.update.assert_not_called()
         self.member_repo.update.assert_not_called()
 
+    def test_update_health_lb_admin_down(self):
+
+        health = {
+            "id": self.FAKE_UUID_1,
+            "listeners": {}}
+
+        fake_lb = mock.MagicMock()
+        fake_lb.enabled = False
+        self.hm.amphora_repo.get_all_lbs_on_amphora.return_value = [fake_lb]
+
+        lb = mock.MagicMock()
+        lb.operating_status.lower.return_value = 'blah'
+        self.amphora_repo.get.load_balancer_id.return_value = self.FAKE_UUID_1
+        self.loadbalancer_repo.get.return_value = lb
+
+        self.hm.update_health(health)
+        self.assertTrue(self.amphora_repo.get.called)
+        self.assertTrue(lb.operating_status.lower.called)
+        self.assertTrue(self.loadbalancer_repo.update.called)
+        self.loadbalancer_repo.update.assert_called_with(
+            self.mock_session(),
+            self.amphora_repo.get().load_balancer_id,
+            operating_status='OFFLINE')
+
+    def test_update_health_lb_admin_up(self):
+
+        health = {
+            "id": self.FAKE_UUID_1,
+            "listeners": {}}
+
+        fake_lb = mock.MagicMock()
+        fake_lb.enabled = True
+        self.hm.amphora_repo.get_all_lbs_on_amphora.return_value = [fake_lb]
+
+        lb = mock.MagicMock()
+        lb.operating_status.lower.return_value = 'blah'
+        self.amphora_repo.get.load_balancer_id.return_value = self.FAKE_UUID_1
+        self.loadbalancer_repo.get.return_value = lb
+
+        self.hm.update_health(health)
+        self.assertTrue(self.amphora_repo.get.called)
+        self.assertTrue(lb.operating_status.lower.called)
+        self.assertTrue(self.loadbalancer_repo.update.called)
+        self.loadbalancer_repo.update.assert_called_with(
+            self.mock_session(),
+            self.amphora_repo.get().load_balancer_id,
+            operating_status='ONLINE')
+
 
 class TestUpdateStatsDb(base.TestCase):
 
     def setUp(self):
         super(TestUpdateStatsDb, self).setUp()
+
+        conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        conf.config(group="health_manager",
+                    event_streamer_driver='queue_event_streamer')
+
         self.sm = update_db.UpdateStatsDb()
         self.event_client = mock.MagicMock()
         self.sm.event_streamer.client = self.event_client
@@ -456,47 +712,98 @@ class TestUpdateStatsDb(base.TestCase):
         self.listener_stats_repo = mock.MagicMock()
         self.sm.listener_stats_repo = self.listener_stats_repo
 
-        self.bytes_in = random.randrange(1000000000)
-        self.bytes_out = random.randrange(1000000000)
-        self.active_conns = random.randrange(1000000000)
-        self.total_conns = random.randrange(1000000000)
         self.loadbalancer_id = uuidutils.generate_uuid()
         self.listener_id = uuidutils.generate_uuid()
 
+        self.listener_stats = data_models.ListenerStatistics(
+            listener_id=self.listener_id,
+            bytes_in=random.randrange(1000000000),
+            bytes_out=random.randrange(1000000000),
+            active_connections=random.randrange(1000000000),
+            total_connections=random.randrange(1000000000),
+            request_errors=random.randrange(1000000000))
+
+        self.sm.get_listener_stats = mock.MagicMock()
+        self.sm.get_listener_stats.return_value = self.listener_stats
+
+        self.loadbalancer_id = uuidutils.generate_uuid()
+        self.amphora_id = uuidutils.generate_uuid()
+        self.listener_id = uuidutils.generate_uuid()
+
+        self.listener = db_models.Listener(
+            load_balancer_id=self.loadbalancer_id)
+
+        self.listener_repo = mock.MagicMock()
+        self.sm.repo_listener = self.listener_repo
+        self.sm.repo_listener.get.return_value = self.listener
+
+        self.loadbalancer_repo = mock.MagicMock()
+        self.sm.repo_loadbalancer = self.loadbalancer_repo
+
+        self.loadbalancer = db_models.LoadBalancer(
+            id=self.loadbalancer_id,
+            listeners=[self.listener])
+        self.loadbalancer_repo.get.return_value = self.loadbalancer
+
     @mock.patch('octavia.db.api.get_session')
     def test_update_stats(self, session):
 
         health = {
-            "id": self.loadbalancer_id,
+            "id": self.amphora_id,
             "listeners": {
-                self.listener_id: {"status": constants.OPEN,
-                                   "stats": {"conns": self.active_conns,
-                                             "totconns": self.total_conns,
-                                             "rx": self.bytes_in,
-                                             "tx": self.bytes_out},
-                                   "pools": {"pool-id-1":
-                                             {"status": constants.UP,
-                                              "members":
-                                              {"member-id-1": constants.ONLINE}
-                                              }
-                                             }
-                                   }}}
+                self.listener_id: {
+                    "status": constants.OPEN,
+                    "stats": {
+                        "ereq": self.listener_stats.request_errors,
+                        "conns": self.listener_stats.active_connections,
+                        "totconns": self.listener_stats.total_connections,
+                        "rx": self.listener_stats.bytes_in,
+                        "tx": self.listener_stats.bytes_out,
+                    },
+                    "pools": {
+                        "pool-id-1": {
+                            "status": constants.UP,
+                            "members": {"member-id-1": constants.ONLINE}
+                        }
+                    }
+                }
+            }
+        }
 
         session.return_value = 'blah'
 
         self.sm.update_stats(health)
 
         self.listener_stats_repo.replace.assert_called_once_with(
-            'blah', self.listener_id, self.loadbalancer_id,
-            bytes_in=self.bytes_in, bytes_out=self.bytes_out,
-            active_connections=self.active_conns,
-            total_connections=self.total_conns)
-        self.event_client.cast.assert_called_once_with(
+            'blah', self.listener_id, self.amphora_id,
+            bytes_in=self.listener_stats.bytes_in,
+            bytes_out=self.listener_stats.bytes_out,
+            active_connections=self.listener_stats.active_connections,
+            total_connections=self.listener_stats.total_connections,
+            request_errors=self.listener_stats.request_errors)
+        self.event_client.cast.assert_any_call(
             {}, 'update_info', container={
                 'info_type': 'listener_stats',
                 'info_id': self.listener_id,
                 'info_payload': {
-                    'bytes_in': self.bytes_in,
-                    'total_connections': self.total_conns,
-                    'active_connections': self.active_conns,
-                    'bytes_out': self.bytes_out}})
+                    'bytes_in': self.listener_stats.bytes_in,
+                    'total_connections':
+                        self.listener_stats.total_connections,
+                    'active_connections':
+                        self.listener_stats.active_connections,
+                    'bytes_out': self.listener_stats.bytes_out,
+                    'request_errors': self.listener_stats.request_errors}})
+
+        self.event_client.cast.assert_any_call(
+            {}, 'update_info',
+            container={
+                'info_type': 'loadbalancer_stats',
+                'info_id': self.loadbalancer_id,
+                'info_payload': {
+                    'bytes_in': self.listener_stats.bytes_in,
+                    'total_connections':
+                        self.listener_stats.total_connections,
+                    'active_connections':
+                        self.listener_stats.active_connections,
+                    'bytes_out': self.listener_stats.bytes_out,
+                    'request_errors': self.listener_stats.request_errors}})
